Boost GIL


Public Types | Public Member Functions | Friends | List of all members
iterator_from_2d< Loc2 > Class Template Reference

Provides 1D random-access navigation to the pixels of the image. Models: PixelIteratorConcept, PixelBasedConcept, HasDynamicXStepTypeConcept. More...

#include <iterator_from_2d.hpp>

Inherits iterator_interface< iterator_from_2d< Loc2 >, std::random_access_iterator_tag, Loc2::value_type, Loc2::reference >.

Public Types

using difference_type = typename parent_t::difference_type
 
using reference = typename parent_t::reference
 
using iterator_category = typename parent_t::iterator_concept
 
using x_iterator = typename Loc2::x_iterator
 
using point_t = typename Loc2::point_t
 

Public Member Functions

std::ptrdiff_t width () const
 
std::ptrdiff_t x_pos () const
 
std::ptrdiff_t y_pos () const
 
bool is_1d_traversable () const
 
x_iterator & x ()
 
 iterator_from_2d (const Loc2 &p, std::ptrdiff_t width, std::ptrdiff_t x=0, std::ptrdiff_t y=0)
 
 iterator_from_2d (const iterator_from_2d &pit)
 
template<typename Loc >
 iterator_from_2d (const iterator_from_2d< Loc > &pit)
 
iterator_from_2doperator= (iterator_from_2d const &other)=default
 
constexpr auto operator* () const noexcept -> reference
 
constexpr auto operator+= (difference_type d) -> iterator_from_2d &
 
constexpr auto operator++ () noexcept -> iterator_from_2d &
 
constexpr auto operator-- () noexcept -> iterator_from_2d &
 
constexpr auto operator++ (int) noexcept -> iterator_from_2d
 
constexpr auto operator-- (int) noexcept -> iterator_from_2d
 
constexpr auto operator- (iterator_from_2d other) const noexcept
 
constexpr bool operator== (iterator_from_2d other) const noexcept
 

Friends

struct boost::stl_interfaces::access
 

Detailed Description

template<typename Loc2>
class boost::gil::iterator_from_2d< Loc2 >

Provides 1D random-access navigation to the pixels of the image. Models: PixelIteratorConcept, PixelBasedConcept, HasDynamicXStepTypeConcept.

Pixels are traversed from the top to the bottom row and from the left to the right within each row


The documentation for this class was generated from the following file: