Release Notes 5.0

Version Compatibility

Vitis™ AI v5.0 and the DPU IP released with the v5.0 branch of this repository are verified as compatible with Vitis, Vivado™ , and PetaLinux version 2025.1. If you are using a previous release of Vitis AI, you should review the version compatibility matrix for that release.

Documentation and Github Repository

  • Migrated core documentation to Github.IO.

  • Incorporated offline HTML documentation for air-gapped users.

  • Restructured user documentation.

  • Restructured repository directory structure for clarity and ease-of-use.

Runtime and Library

  • Upgraded to enable VART base on XRT 2025.1

  • For vitis_ai_library are not enabled in this release, If you want to use vitis_ai_library you can refer to previous versions.

DPU IP - Zynq Ultrascale+ DPUCZDX8G

  • Upgraded to enable Vivado™ and Vitis 2025.1 release

DPU IP - Versal AIE Targets DPUCVDX8G

  • Upgraded to enable Vivado and Vitis 2025.1 release

AMD, the AMD Arrow logo, and combinations thereof are trademarks of Advanced Micro Devices, Inc.