skyllh.p1 package#
Submodules#
skyllh.p1.detector_model module#
This module defines the P-ONE detector model class.
- class skyllh.p1.detector_model.PONEDetectorModel(**kwargs)#
Bases:
DetectorModelDefinition of the P-ONE detector.
For the time being we use the position of STRAW-a. As depth we use the sea floor at -2658m and a detector half height of 500m.
Creates a new DetectorModel instance.
- Parameters:
name (str) – The name of the detector model.
location (instance of astropy.coordinates.EarthLocation) – The Earth location of the detector.
- __annotations_cache__ = {}#
- __firstlineno__ = 20#
- __static_attributes__ = ()#