skyllh.plotting.i3 package#

Submodules#

skyllh.plotting.i3.backgroundpdf module#

Plotting module to plot IceCube specific background PDF objects.

class skyllh.plotting.i3.backgroundpdf.BackgroundI3SpatialPDFPlotter(tdm, pdf)#

Bases: object

Plotter class to plot an BackgroundI3SpatialPDF object.

Creates a new plotter object for plotting an BackgroundI3SpatialPDF object.

Parameters:
__annotations_cache__ = {}#
__firstlineno__ = 32#
__static_attributes__ = ('_pdf', '_tdm', 'pdf', 'tdm')#
property pdf#

The PDF object to plot.

property tdm#

The TrialDataManager that provides the data for the PDF evaluation.

plot(src_hypo_group_manager, axes)#

Plots the spatial PDF. It uses the sin(dec) binning of the PDF to propperly represent the resolution of the PDF in the drawing.

Parameters:
  • src_hypo_group_manager (instance of SourceHypoGroupManager) – The instance of SourceHypoGroupManager that defines the source hypotheses.

  • axes (mpl.axes.Axes) – The matplotlib Axes object on which the PDF should get drawn to.

Returns:

img (instance of mpl.AxesImage) – The AxesImage instance showing the PDF image.