solutions.node

Function Function object.
Node Node object.
calc_node_depth Calculate the depth of nodes which is following under the target node.
children_checker
copy_node Copy function :param node: node object, target node to copy.
copy_nodes_along_graph Copy node object from root to the target node based on ``graph’’ This method differs from deepcopy(root) in that it copies only the nodes along ``graph’‘.:param graph: list of ``(i, Node object)’’ where ``i’’ is the index of the next node of graph and ``Node object’’ is the parent node.This graph is obtained by using ``get_parent_node’‘.:return: the index of target node node in the parent, copied the target node object and copied root object..
func_id_checker
get_all_node function for getting all node in the solution
get_all_nonterminal_nodes function for getting all terminal node in the solution
get_all_terminal_nodes function for getting all terminal node in the solution
get_all_terminal_points function for getting all terminal points in the solution this function is for crossover in MLPS-GP
get_graph_to_target function for searching a graph from root node to a target node.
get_n_children Get the number of children of function that id = func_id.
get_parent_node function for searching parent node of target node.
n_children_checker
node_array_equal
node_checker
node_equal Function for comparing two nodes.
nodes_checker
set_children Setter for children to node :param node: node object.
set_id Function for setting id to node object.