solutions.solution

Solution
_calc_solution_depth
_calc_solution_n_nodes
copy_solution Copy the target solution :param solution: solution object.
is_solution_in_pop check the tree of ``solution’’ is in ``population’’ or not :param solution: Solution object :param population: list of Solution object :param as_tree: If as_tree is true, solution is compared by all the nodes’ structure Otherwise, compared by object id.:return: bool.
replace_node Replace a node in a solution by another node.
select_random_points Obtain k points in the solution at random.
set_previous_fitness
set_solution_depth
set_solution_n_nodes
solution_checker
solution_equal
solutions_checker