skyllh.datasets.i3 package

Submodules

skyllh.datasets.i3.PublicData_10y_ps module

skyllh.datasets.i3.PublicData_10y_ps.create_dataset_collection(cfg, base_path=None, sub_path_fmt=None)

Defines the dataset collection for IceCube’s 10-year point-source public data, which is available at https://doi.org/10.7910/DVN/VKL316. Version 2.0 renames two event files (IC79_exp.csv and IC86_VII_exp.csv) so that they have a naming scheme consistent with the rest of the data files.

Parameters:
  • cfg (instance of Config) – The instance of Config holding the local configuration.

  • base_path (str | None) – The base path of the data files. The actual path of a data file is assumed to be of the structure <base_path>/<sub_path>/<file_name>. If None, cfg['repository']['base_path'] is used, which defaults to ~/.cache/skyllh.

  • sub_path_fmt (str | None) – The sub path format of the data files of the public data sample. If None, use the default sub path format ‘icecube_10year_ps’.

Returns:

dsc (DatasetCollection) – The dataset collection containing all the seasons as individual I3Dataset objects.

skyllh.datasets.i3.PublicData_10y_ps_wMC module

skyllh.datasets.i3.PublicData_10y_ps_wMC.create_dataset_collection(cfg, base_path=None, sub_path_fmt=None)

Defines the dataset collection for IceCube’s 10-year point-source public data, which is available at http://icecube.wisc.edu/data-releases/20210126_PS-IC40-IC86_VII.zip, and adds monte-carlo files.

Parameters:
  • cfg (instance of Config) – The instance of Config holding the local configuration.

  • base_path (str | None) – The base path of the data files. The actual path of a data file is assumed to be of the structure <base_path>/<sub_path>/<file_name>. If None, cfg['repository']['base_path'] is used, which defaults to ~/.cache/skyllh.

  • sub_path_fmt (str | None) – The sub path format of the data files of the public data sample. If None, use the default sub path format ‘icecube_10year_ps’.

Returns:

dsc (DatasetCollection) – The dataset collection containing all the seasons as individual I3Dataset objects.

skyllh.datasets.i3.PublicData_14y_ps module

skyllh.datasets.i3.PublicData_14y_ps.create_dataset_collection(cfg, base_path=None, sub_path_fmt=None)

Defines the dataset collection for IceCube’s 14-year point-source public data, which is available at https://doi.org/10.7910/DVN/MMIIZA.

Note that compared to the previous data release: - a finer declination binning is provided for the 14yr dataset, thus

providing a more accurate description of the detector response variations across the sky;

  • the detector configuration and selection were homogenized for all data taking seasons using the full, 86-string detector configuration (IC86).

Parameters:
  • cfg (instance of Config) – The instance of Config holding the local configuration.

  • base_path (str | None) – The base path of the data files. The actual path of a data file is assumed to be of the structure <base_path>/<sub_path>/<file_name>. If None, cfg['repository']['base_path'] is used, which defaults to ~/.cache/skyllh.

  • sub_path_fmt (str | None) – The sub path format of the data files of the public data sample. If None, use the default sub path format ‘icecube_14year_ps’.

Returns:

dsc (DatasetCollection) – The dataset collection containing all the seasons as individual I3Dataset objects.

skyllh.datasets.i3.TestData module

skyllh.datasets.i3.TestData.create_dataset_collection(cfg, base_path=None, sub_path_fmt=None)

Defines a dataset collection with a test dataset.

Parameters:
  • cfg (instance of Config) – The instance of Config holding the local configuration.

  • base_path (str | None) – The base path of the data files. The actual path of a data file is assumed to be of the structure <base_path>/<sub_path>/<file_name>. If None, cfg['repository']['base_path'] is used, which defaults to ~/.cache/skyllh.

  • sub_path_fmt (str | None) – The sub path format of the data files of the public data sample. If None, use the default sub path format ‘testdata’.

Returns:

dsc (DatasetCollection) – The dataset collection containing all the seasons as individual I3Dataset objects.