skyllh.plotting.core package¶
Submodules¶
skyllh.plotting.core.pdfratio module¶
Plotting module for core PDF ratio objects.
- class skyllh.plotting.core.pdfratio.SigOverBkgPDFRatioPlotter(tdm: TrialDataManager, pdfratio: SigOverBkgPDFRatio)¶
Bases:
objectPlotter class to plot a SigOverBkgPDFRatio object.
Creates a new plotter object for plotting a SigOverBkgPDFRatio object.
- Parameters:
tdm – The instance of TrialDataManager that provides the data for the PDF ratio evaluation.
pdfratio – The PDF ratio object to plot.
- __annotate_func__ = None¶
- __annotations_cache__ = {}¶
- __firstlineno__ = 20¶
- __static_attributes__ = ('_pdfratio', '_tdm', 'pdfratio', 'tdm')¶
- property pdfratio¶
The PDF ratio object to plot.
- property tdm¶
The TrialDataManager that provides the data for the PDF evaluation.
- plot(src_hypo_group_manager: SourceHypoGroupManager, pmm: ParameterModelMapper, src_params_recarray: ndarray, axes: Axes, source_idx: int | None = None, log: bool = True, **kwargs) AxesImage¶
Plots the spatial PDF ratio. If the signal PDF depends on the source, source_idx specifies the index of the source for which the PDF should get plotted.
- Parameters:
src_hypo_group_manager – The instance of SourceHypoGroupManager that defines the source hypotheses.
axes – The matplotlib Axes object on which the PDF ratio should get drawn to.
source_idx – The index of the source for which the PDF ratio should get plotted. If set to None and the signal PDF depends on the source, index 0 will be used.
log – Whether to use a logarithmic color scale for the PDF ratio image.
Arguments (Additional Keyword)
----------------------------
mpl.imshow (Any additional keyword arguments will be passed to the)
function.
- Returns:
img – The AxesImage instance showing the PDF ratio image.
skyllh.plotting.core.signalpdf module¶
- class skyllh.plotting.core.signalpdf.SignalSpatialPDFPlotter(tdm: TrialDataManager, pdf, **kwargs)¶
Bases:
objectPlotter class to plot spatial signal PDF object.
Creates a new plotter object for plotting a spatial signal PDF object.
- Parameters:
tdm – The instance of TrialDataManager that provides the data for the PDF evaluation.
pdf – The PDF object to plot.
- __annotate_func__ = None¶
- __annotations_cache__ = {}¶
- __firstlineno__ = 28¶
- __static_attributes__ = ('_pdf', '_tdm', 'pdf', 'tdm')¶
- property pdf¶
The PDF object to plot.
- property tdm¶
The TrialDataManager that provides the data for the PDF evaluation.
- plot(src_hypo_group_manager: SourceHypoGroupManager, pmm: ParameterModelMapper, axes: Axes, source_idx: int | None = None, sin_dec: bool = True, log=True, **kwargs)¶
Plots the signal spatial PDF for the specified source.
- Parameters:
axes – The matplotlib Axes object on which the PDF ratio should get drawn to.
source_idx – The index of the source for which the PDF ratio should get plotted. If set to None and the signal PDF depends on the source, index 0 will be used.
sin_dec – Flag if the plot should be made in right-ascention vs. declination (False), or in right-ascention vs. sin(declination) (True).
Arguments (Additional Keyword)
----------------------------
mpl.imshow (Any additional keyword arguments will be passed to the)
function.
- Returns:
img – The AxesImage instance showing the PDF ratio image.