skyllh.plotting.i3 package#

Submodules#

skyllh.plotting.i3.backgroundpdf module#

Plotting module to plot IceCube specific background PDF objects.

class skyllh.plotting.i3.backgroundpdf.BackgroundI3SpatialPDFPlotter(tdm: TrialDataManager, pdf: BackgroundI3SpatialPDF)#

Bases: object

Plotter class to plot an BackgroundI3SpatialPDF object.

Creates a new plotter object for plotting an BackgroundI3SpatialPDF object.

Parameters:
  • tdm – The instance of TrialDataManager that provides the data for the PDF evaluation.

  • pdf – The PDF object to plot.

__annotations_cache__ = {}#
__firstlineno__ = 31#
__static_attributes__ = ('_pdf', '_tdm', 'pdf', 'tdm')#
property pdf#

The PDF object to plot.

property tdm#

The TrialDataManager that provides the data for the PDF evaluation.

plot(src_hypo_group_manager: SourceHypoGroupManager, pmm: ParameterModelMapper, axes: Axes)#

Plots the spatial PDF. It uses the sin(dec) binning of the PDF to propperly represent the resolution of the PDF in the drawing.

Parameters:
  • src_hypo_group_manager – The instance of SourceHypoGroupManager that defines the source hypotheses.

  • axes – The matplotlib Axes object on which the PDF should get drawn to.

Returns:

img – The AxesImage instance showing the PDF image.

skyllh.plotting.i3.pdf module#

Plotting module to plot IceCube specific PDF objects.

class skyllh.plotting.i3.pdf.I3EnergyPDFPlotter(tdm: TrialDataManager, pdf: I3EnergyPDF)#

Bases: object

Plotter class to plot an I3EnergyPDF object.

Creates a new plotter object for plotting an I3EnergyPDF object.

Parameters:
  • tdm – The instance of TrialDataManager that provides the data for the PDF evaluation.

  • pdf – The PDF object to plot.

__annotations_cache__ = {}#
__firstlineno__ = 33#
__static_attributes__ = ('_pdf', '_tdm', 'pdf', 'tdm')#
property pdf#

The PDF object to plot.

property tdm#

The TrialDataManager that provides the data for the PDF evaluation.

plot(src_hypo_group_manager: SourceHypoGroupManager, pmm: ParameterModelMapper, axes: Axes, **kwargs)#

Plots the PDF object.

Parameters:
  • src_hypo_group_manager – The instance of SourceHypoGroupManager that defines the source hypotheses.

  • pmm – The instance of ParameterModelMapper that maps the global parameters to the local model parameters.

  • axes – The matplotlib Axes object on which the PDF should get drawn to.

  • Arguments (Additional Keyword)

  • ----------------------------

  • mpl.imshow (Any additional keyword arguments will be passed to the)

  • function.

Returns:

img – The AxesImage instance showing the PDF ratio image.

skyllh.plotting.i3.pdfratio module#

Plotting module to plot IceCube specific PDF ratio objects.

class skyllh.plotting.i3.pdfratio.SplinedI3EnergySigSetOverBkgPDFRatioPlotter(tdm: TrialDataManager, pdfratio: SplinedI3EnergySigSetOverBkgPDFRatio)#

Bases: object

Plotter class to plot an SplinedI3EnergySigSetOverBkgPDFRatio object.

Creates a new plotter object for plotting an SplinedI3EnergySigSetOverBkgPDFRatio object.

Parameters:
  • tdm – The instance of TrialDataManager that provides the data for the PDF ratio evaluation.

  • pdfratio – The PDF ratio object to plot.

__annotations_cache__ = {}#
__firstlineno__ = 22#
__static_attributes__ = ('_pdfratio', '_tdm', 'pdfratio', 'tdm')#
property pdfratio#

The PDF ratio object to plot.

property tdm#

The TrialDataManager that provides the data for the PDF evaluation.

plot(src_hypo_group_manager: SourceHypoGroupManager, pmm: ParameterModelMapper, src_params_recarray: ndarray, axes: Axes, **kwargs) AxesImage#

Plots the PDF ratio for the given set of fit paramater values.

Parameters:
  • src_hypo_group_manager – The instance of SourceHypoGroupManager that defines the source hypotheses.

  • pmm – The instance of ParameterModelMapper that maps the global parameters to the local model parameters.

  • src_params_recarray – The numpy record ndarray holding the parameter names and values of all sources.

  • axes – The matplotlib Axes object on which the PDF ratio should get drawn to.

  • Arguments (Additional Keyword)

  • ----------------------------

  • mpl.imshow (Any additional keyword arguments will be passed to the)

  • function.

Returns:

img – The AxesImage instance showing the PDF ratio image.